import { isEmpty } from 'lodash'
import type { IASMAAppsObject as IAsmaAppsObject } from './interfaces'
import { loadMicroApp, type RegistrableApp } from './qiankun'

import { registerMicroApps } from './qiankun/apis'
/**
 *@readonly do not update directly use setAppsObject() in stead
 */
let asmaRegistrableApps: IAsmaAppsObject | undefined
export function getAsmaRegistrableApps() {
    return asmaRegistrableApps
}
/**
 * In dev mode it  set ImportMapOverrides app
 *
 * call this function before render App() method
 *
 * @param registrable_apps IAsmaAppsObject
 * @param devtools bool
 */
export async function setAsmaRegistrableApps(registrable_apps?: IAsmaAppsObject, devtools?: boolean) {
    if (!registrable_apps) {
        devtools && console.warn('No registrable apps detected!')

        return
    }

    if (devtools) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        const _ = await import('qiankun-overrides')

        const imo:
            | {
                  setDefaultMap: (apps: RegistrableApp<{}>[]) => void
                  getCurrentQiankunMap: () => IAsmaAppsObject | undefined
              }
            | undefined = window.importMapOverrides

        imo?.setDefaultMap(Object.values(registrable_apps))

        asmaRegistrableApps = imo?.getCurrentQiankunMap()
    } else {
        asmaRegistrableApps = registrable_apps
    }

    if (!window.__ASMA__QIANKUN__SHELL__) {
        window.__ASMA__QIANKUN__SHELL__ = {}
    }

    window.__ASMA__QIANKUN__SHELL__.loadMicroApp = loadMicroApp
}

/**
 * before registerAsmaMicroApps one need to call setAsmaRegistrableApps to ensure microappregistration!
 * call this method after render App() method.
 */
export async function registerAsmaMicroApps() {
    if (asmaRegistrableApps && !isEmpty(asmaRegistrableApps)) {
        registerMicroApps(
            Object.values(asmaRegistrableApps).filter((app) => app.activeRule !== 'component-only'),
            {
                beforeLoad: async (app) => {
                    console.log('[LifeCycle] before load', 'color: green;', app.name)
                },

                beforeMount: async (app) => {
                    console.log('[LifeCycle] before mount', 'color: green;', app.name)
                },

                afterUnmount: async (app) => {
                    console.log('[LifeCycle] after unmount', 'color: green;', app.name)
                },
            },
        )
    }
}
